package ipspatcher

import java.awt.Window
import java.io.File
import java.io.FileReader
import java.io.FileWriter
import java.io.IOException
import java.io.RandomAccessFile
import javax.swing.JDialog
import javax.swing.JOptionPane

class IPSTool {
    internal var parent: Window
    internal var DlgProgress: JDialog? = null
    internal var OrigIdx = -1
    internal var PatchIdx = -1
    internal var OrigFile: RandomAccessFile
    internal var PatchFile: RandomAccessFile
    internal var LogFile: RandomAccessFile
    internal var Address: Int = 0
    internal var NumBytes: Int = 0
    internal var RLEBytes: Int = 0
    internal var TotalChunks: Int = 0
    internal var RLEByte: Char = ' '
    internal var Buffer: ByteArray
    internal var log: Boolean = false
    internal var backup: Boolean = false

    @Throws(IOException::class)
    constructor(p: Window, file: File, patch: File, filezip: Boolean, patchzip: Boolean, Log: Boolean, Backup: Boolean) {
        this.init(p, file, patch, Log, Backup)
    }

    @Throws(IOException::class)
    constructor(p: Window, file: File, patch: File, Log: Boolean, Backup: Boolean) {
        this.init(p, file, patch, Log, Backup)
    }

    @Throws(IOException::class)
    fun init(p: Window, file: File, patch: File, Log: Boolean, Backup: Boolean) {
        this.PatchFile = RandomAccessFile(patch, "r")
        this.OrigFile = RandomAccessFile(file, "rw")
        this.TotalChunks = 0
        this.Address = 0
        this.NumBytes = 0
        this.RLEBytes = 0
        this.log = Log
        this.parent = p
        if (this.log) {
            this.LogFile = RandomAccessFile(patch.parent + File.separatorChar + "log.txt", "rw")
        }

        this.backup = Backup
        if (this.backup) {
            val outputFile = File(file.path + ".bak")
            val `in` = FileReader(file)
            val out = FileWriter(outputFile.path, false)

            var c: Int
            while ((c = `in`.read()) != -1) {
                out.write(c)
            }

            `in`.close()
            out.close()
        }

    }

    @Throws(IOException::class)
    internal fun GetNumBytes() {
        val b1 = this.PatchFile.readUnsignedByte()
        val b2 = this.PatchFile.readUnsignedByte()
        this.NumBytes = b2
        this.NumBytes += b1 * 256
        if (this.NumBytes == 0) {
            this.GetRLE()
        }

    }

    @Throws(IOException::class)
    internal fun GetAddress(): Boolean {
        if (this.PatchIdx == -1) {
            val a1 = this.PatchFile.readUnsignedByte()
            val a2 = this.PatchFile.readUnsignedByte()
            val a3 = this.PatchFile.readUnsignedByte()
            if (a1 == 69 && a2 == 79 && a3 == 70) {
                return false
            }

            ++this.TotalChunks
            this.Address = 0
            this.Address = a3
            this.Address += a2 * 256
            this.Address += a1 * 65536
        }

        return true
    }

    @Throws(IOException::class)
    internal fun isaIPS(): Boolean {
        this.Buffer = ByteArray(5)
        this.PatchFile.read(this.Buffer)
        return this.Buffer[0].toInt() == 80 && this.Buffer[1].toInt() == 65 && this.Buffer[2].toInt() == 84 && this.Buffer[3].toInt() == 67 && this.Buffer[4].toInt() == 72
    }

    @Throws(IOException::class)
    internal fun GetChunk() {
        this.Buffer = ByteArray(this.NumBytes)
        this.PatchFile.read(this.Buffer)
    }

    @Throws(IOException::class)
    internal fun ApplyPatch() {
        this.OrigFile.seek(this.Address.toLong())
        this.OrigFile.write(this.Buffer, 0, this.NumBytes)
    }

    @Throws(IOException::class)
    internal fun ApplyRLE() {
        this.OrigFile.seek(this.Address.toLong())

        for (i in 0 until this.NumBytes) {
            this.OrigFile.write(this.RLEByte.toInt())
        }

    }

    @Throws(IOException::class)
    internal fun GetRLE() {
        val r1 = this.PatchFile.readUnsignedByte()
        val r2 = this.PatchFile.readUnsignedByte()
        val rb = this.PatchFile.readUnsignedByte()
        this.RLEBytes = r2
        this.RLEBytes += r1 * 256
        this.RLEByte = rb.toChar()
    }

    fun patch() {
        try {
            if (this.isaIPS()) {
                if (this.log) {
                    this.LogFile.writeBytes("JIPS 3.0 Logfile\nTo have news about JIPS and other ZeroSoft Projects\nvisit: http://zerosoft.zophar.net\n\n")
                }

                while (this.GetAddress()) {
                    this.GetNumBytes()
                    if (this.NumBytes != 0) {
                        this.GetChunk()
                    }

                    if (this.NumBytes != 0) {
                        this.ApplyPatch()
                        if (this.log) {
                            this.LogFile.writeBytes("Chunk number: " + Integer.toString(this.TotalChunks) + " Offset: " + Integer.toHexString(this.Address) + " Bytes: " + Integer.toHexString(this.NumBytes) + "\n")
                        }
                    } else {
                        this.ApplyRLE()
                        if (this.log) {
                            this.LogFile.writeBytes("Chunk number: " + Integer.toString(this.TotalChunks) + " Offset: " + Integer.toHexString(this.Address) + " Bytes: " + Integer.toHexString(this.NumBytes) + " RLE char: " + Integer.toHexString(this.RLEByte.toInt()) + " Lenght: " + Integer.toString(this.RLEBytes) + "\n")
                        }
                    }
                }

                JOptionPane.showMessageDialog(this.parent, "Patching Complete!\nPatches applied: " + this.TotalChunks)
            }
        } catch (var2: IOException) {
            JOptionPane.showMessageDialog(this.parent, "An error occurred during patching:\n" + var2.message)
        }

    }

    companion object {
        internal var IPSE_Patch = 1
        internal var IPSE_File = 2
    }
}
